@@ -65,13 +65,13 @@ typedef enum {
 
 typedef enum {
 	PDF_GRAPHIC_MODE_GRAPHICS,
-	PDF_GRAPHIC_MODE_TEXT,
+	PDF_GRAPHIC_MODE_TEXT
 } PdfGraphicMode;
 
 typedef enum {
 	PDF_COLOR_MODE_DEVICEGRAY,
 	PDF_COLOR_MODE_DEVICERGB,
-	PDF_COLOR_MODE_DEVICECMYK,
+	PDF_COLOR_MODE_DEVICECMYK
 }PdfColorModes;
 
 typedef enum {
@@ -179,6 +179,7 @@ static gchar *gnome_print_pdf_id_new (GnomePrintPdf *pdf);
 
 static gint  gnome_print_pdf_graphic_mode_set (GnomePrintPdf *pdf, PdfGraphicMode mode);
 static gint  gnome_print_pdf_page_fprintf (GnomePrintPdf *pdf, const char *format, ...);
+static gint  gnome_print_pdf_page_print_double (GnomePrintPdf *pdf, const gchar *format, gdouble x);
 
 static GnomePrintContextClass *parent_class;
 
@@ -516,8 +517,24 @@ gnome_print_pdf_image (GnomePrintContext *pc, const gdouble *ctm, const guchar *
 	else
 		page->used_color_images = TRUE;
 	gnome_print_pdf_page_fprintf (pdf, "q" EOL);
-	gnome_print_pdf_page_fprintf (pdf, "%g %g %g %g %g %g cm" EOL,
-				      ctm[0], ctm[1], ctm[2], ctm[3], ctm[4], ctm[5]);
+	gnome_print_pdf_page_print_double (pdf, "%g",
+				      ctm[0]);
+	gnome_print_pdf_page_fprintf (pdf, " ");
+	gnome_print_pdf_page_print_double (pdf, "%g",
+				      ctm[1]);
+	gnome_print_pdf_page_fprintf (pdf, " ");
+	gnome_print_pdf_page_print_double (pdf, "%g",
+				      ctm[2]);
+	gnome_print_pdf_page_fprintf (pdf, " ");
+	gnome_print_pdf_page_print_double (pdf, "%g",
+				      ctm[3]);
+	gnome_print_pdf_page_fprintf (pdf, " ");
+	gnome_print_pdf_page_print_double (pdf, "%g",
+				      ctm[4]);
+	gnome_print_pdf_page_fprintf (pdf, " ");
+	gnome_print_pdf_page_print_double (pdf, "%g",
+				      ctm[5]);
+	gnome_print_pdf_page_fprintf (pdf, " cm" EOL);
 	gnome_print_pdf_page_fprintf (pdf, "0 0 m" EOL);
 	gnome_print_pdf_page_fprintf (pdf, "/Im%d Do" EOL, object_num);
 	gnome_print_pdf_page_fprintf (pdf, "Q" EOL);
@@ -591,11 +608,20 @@ gnome_print_pdf_glyphlist (GnomePrintContext *pc, const gdouble *ctm, GnomeGlyph
 
 		memcpy (Tm, ctm, 6 * sizeof (gdouble));
 		art_affine_scale (Tm, font->size, font->size);
-		gnome_print_pdf_page_fprintf (pdf,
-					      "%f %f %f %f %f %f Tm" EOL,
-					      Tm [0], Tm [1], Tm [2], Tm [3],
-					      dx + pgl->glyphs[ps->start].x,
-					      dy + pgl->glyphs[ps->start].y);
+		
+		gnome_print_pdf_page_print_double (pdf, "%g", Tm [0]);
+		gnome_print_pdf_page_fprintf (pdf, " ");
+		gnome_print_pdf_page_print_double (pdf, "%g", Tm [1]);
+		gnome_print_pdf_page_fprintf (pdf, " ");
+		gnome_print_pdf_page_print_double (pdf, "%g", Tm [2]);
+		gnome_print_pdf_page_fprintf (pdf, " ");
+		gnome_print_pdf_page_print_double (pdf, "%g", Tm [3]);
+		gnome_print_pdf_page_fprintf (pdf, " ");
+		gnome_print_pdf_page_print_double (pdf, "%g", dx + pgl->glyphs[ps->start].x);
+		gnome_print_pdf_page_fprintf (pdf, " ");
+		gnome_print_pdf_page_print_double (pdf, "%g", dy + pgl->glyphs[ps->start].y);
+		gnome_print_pdf_page_fprintf (pdf, " Tm" EOL);
+
 		/* Build string */
 		ret = gnome_print_pdf_page_fprintf (pdf, "(");
 		
@@ -618,9 +644,15 @@ gnome_print_pdf_glyphlist (GnomePrintContext *pc, const gdouble *ctm, GnomeGlyph
 		ret = gnome_print_pdf_page_fprintf (pdf, "[");
 		g_return_val_if_fail (ret >= 0, ret);
 		for (i = ps->start + 1; i < ps->start + ps->length; i++) {
-			ret = gnome_print_pdf_page_fprintf (pdf, "%g %g ",
-							    pgl->glyphs[i].x - pgl->glyphs[i-1].x,
-							    pgl->glyphs[i].y - pgl->glyphs[i-1].y);
+			ret = gnome_print_pdf_page_print_double (pdf, "%g",
+				     pgl->glyphs[i].x - pgl->glyphs[i-1].x);
+			g_return_val_if_fail (ret >= 0, ret);
+			ret = gnome_print_pdf_page_fprintf (pdf, " ");
+			g_return_val_if_fail (ret >= 0, ret);
+			ret = gnome_print_pdf_page_print_double (pdf, "%g",
+				     pgl->glyphs[i].y - pgl->glyphs[i-1].y);
+			g_return_val_if_fail (ret >= 0, ret);
+			ret = gnome_print_pdf_page_fprintf (pdf, " ");
 			g_return_val_if_fail (ret >= 0, ret);
 		}
 		ret = gnome_print_pdf_page_fprintf (pdf, "0 0] ");
@@ -1187,12 +1219,15 @@ gnome_print_pdf_set_line (GnomePrintPdf *pdf)
 
 	if (gp_gc_get_line_flag (ctx->gc) == GP_GC_FLAG_CLEAR)
 		return GNOME_PRINT_OK;
-
-	gnome_print_pdf_page_fprintf (pdf, "%g w %d J %d j %g M" EOL,
-				      gp_gc_get_linewidth (ctx->gc),
+	gnome_print_pdf_page_print_double (pdf, "%g", 
+					   gp_gc_get_linewidth (ctx->gc));
+	gnome_print_pdf_page_fprintf (pdf, " w %d J %d j ",
 				      gp_gc_get_linecap (ctx->gc),
 				      gp_gc_get_linejoin (ctx->gc),
 				      gp_gc_get_miterlimit (ctx->gc));
+	gnome_print_pdf_page_print_double (pdf, "%g", 
+					   gp_gc_get_miterlimit (ctx->gc));
+	gnome_print_pdf_page_fprintf (pdf, " M" EOL);
 	gp_gc_set_line_flag (ctx->gc, GP_GC_FLAG_CLEAR);
 
 	return GNOME_PRINT_OK;
@@ -1212,9 +1247,14 @@ gnome_print_pdf_set_dash (GnomePrintPdf *pdf)
 
 	dash = gp_gc_get_dash (ctx->gc);
 	gnome_print_pdf_page_fprintf (pdf, "[");
-	for (i = 0; i < dash->n_dash; i++)
-		gnome_print_pdf_page_fprintf (pdf, " %g", dash->dash[i]);
-	gnome_print_pdf_page_fprintf (pdf, "]%g d" EOL, dash->n_dash > 0 ? dash->offset : 0.0);
+	for (i = 0; i < dash->n_dash; i++) {
+		gnome_print_pdf_page_fprintf (pdf, " ");
+		gnome_print_pdf_page_print_double (pdf, "%g", dash->dash[i]);
+	}
+	gnome_print_pdf_page_fprintf (pdf, "]");
+	gnome_print_pdf_page_print_double (pdf, "%g", 
+				 dash->n_dash > 0 ? dash->offset : 0.0);
+	gnome_print_pdf_page_fprintf (pdf, " d" EOL);
 	gp_gc_set_dash_flag (ctx->gc, GP_GC_FLAG_CLEAR);
 
 	return GNOME_PRINT_OK;
@@ -1294,7 +1334,8 @@ gnome_print_pdf_font_print_widths (GnomePrintPdf *pdf, GnomePrintPdfFont *font)
 			gnome_print_pdf_fprintf (pdf, EOL);
 			col = 0;
 		}
-		col += gnome_print_pdf_fprintf (pdf, "%g ", point.x);
+		col += gnome_print_pdf_print_double (pdf, "%g", point.x);
+		col += gnome_print_pdf_fprintf (pdf, " ");
 		i++;
 	}
 	
@@ -1465,14 +1506,22 @@ gnome_print_pdf_font_print_descriptor (GnomePrintPdf *pdf, GnomePrintPdfFont *fo
 				 "/CapHeight %d" EOL
 				 "/Descent %d" EOL
 				 "/Flags %d" EOL
-				 "/FontBBox [%g %g %g %g]" EOL
+				 "/FontBBox [",
+				 ascent, capheight,
+				 descent * -1, flags);
+	gnome_print_pdf_print_double (pdf, "%g", new_rect.x0);
+	gnome_print_pdf_fprintf (pdf, " ");
+	gnome_print_pdf_print_double (pdf, "%g", new_rect.y0);
+	gnome_print_pdf_fprintf (pdf, " ");
+	gnome_print_pdf_print_double (pdf, "%g", new_rect.x1);
+	gnome_print_pdf_fprintf (pdf, " ");
+	gnome_print_pdf_print_double (pdf, "%g", new_rect.y1);
+	gnome_print_pdf_fprintf (pdf,
+				 "]" EOL
 				 "/FontName /%s" EOL
 				 "/ItalicAngle %d" EOL
 				 "/StemV %d" EOL
 				 "/XHeight %d" EOL,
-				 ascent, capheight,
-				 descent * -1, flags,
-				 new_rect.x0, new_rect.y0, new_rect.x1, new_rect.y1,
 				 gnome_font_face_get_ps_name (font->face),
 				 italic_angle, stemv, xheight);
 
@@ -1648,23 +1697,40 @@ gnome_print_pdf_print_bpath (GnomePrintPdf *pdf, const ArtBpath *bpath)
 				gnome_print_pdf_page_fprintf (pdf, "h" EOL);
 			closed = FALSE;
 			started = FALSE;
-			gnome_print_pdf_page_fprintf (pdf, "%g %g m" EOL, bpath->x3, bpath->y3);
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->x3);
+			gnome_print_pdf_page_fprintf (pdf, " ");
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->y3);
+			gnome_print_pdf_page_fprintf (pdf, " m" EOL);
 			break;
 		case ART_MOVETO:
 			if (started && closed)
 				gnome_print_pdf_page_fprintf (pdf, "h" EOL);
 			closed = TRUE;
 			started = TRUE;
-			gnome_print_pdf_page_fprintf (pdf, "%g %g m" EOL, bpath->x3, bpath->y3);
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->x3);
+			gnome_print_pdf_page_fprintf (pdf, " ");
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->y3);
+			gnome_print_pdf_page_fprintf (pdf, " m" EOL);
 			break;
 		case ART_LINETO:
-			gnome_print_pdf_page_fprintf (pdf, "%g %g l" EOL, bpath->x3, bpath->y3);
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->x3);
+			gnome_print_pdf_page_fprintf (pdf, " ");
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->y3);
+			gnome_print_pdf_page_fprintf (pdf, " l" EOL);
 			break;
 		case ART_CURVETO:
-			gnome_print_pdf_page_fprintf (pdf, "%g %g %g %g %g %g c" EOL,
-						      bpath->x1, bpath->y1,
-						      bpath->x2, bpath->y2,
-						      bpath->x3, bpath->y3);
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->x1);
+			gnome_print_pdf_page_fprintf (pdf, " ");
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->y1);
+			gnome_print_pdf_page_fprintf (pdf, " ");
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->x2);
+			gnome_print_pdf_page_fprintf (pdf, " ");
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->y2);
+			gnome_print_pdf_page_fprintf (pdf, " ");
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->x3);
+			gnome_print_pdf_page_fprintf (pdf, " ");
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->y3);
+			gnome_print_pdf_page_fprintf (pdf, " c" EOL);
 			break;
 		default:
 			g_warning ("Path structure is corrupted");
@@ -1739,18 +1805,11 @@ gnome_print_pdf_fprintf (GnomePrintPdf *pdf, const char *format, ...)
 {
  	va_list arguments;
  	gchar *text;
-	gchar *oldlocale;
 	gint len;
 
-	oldlocale = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
-#warning: We should not be using setlocale here
-
  	va_start (arguments, format);
  	text = g_strdup_vprintf (format, arguments);
  	va_end (arguments);
-	setlocale (LC_NUMERIC, oldlocale);
-	g_free (oldlocale);
 
 	len = strlen (text);
 	gnome_print_transport_write (GNOME_PRINT_CONTEXT (pdf)->transport, text, len);
@@ -1760,6 +1819,24 @@ gnome_print_pdf_fprintf (GnomePrintPdf *pdf, const char *format, ...)
 	return len;
 }
 
+gint 
+gnome_print_pdf_print_double (GnomePrintPdf *pdf, const gchar *format, gdouble x)
+{
+ 	gchar *text;
+	gint len;
+
+	text = g_new (gchar, G_ASCII_DTOSTR_BUF_SIZE);
+	g_ascii_formatd (text, G_ASCII_DTOSTR_BUF_SIZE, format, x);
+
+	len = strlen (text);
+	gnome_print_transport_write (GNOME_PRINT_CONTEXT (pdf)->transport, text, len);
+	pdf->offset += len;
+	g_free (text);
+
+	return len;
+}
+
+
 gint
 gnome_print_pdf_print_sized (GnomePrintPdf *pdf, const char *content, gint len)
 {
@@ -1770,27 +1847,12 @@ gnome_print_pdf_print_sized (GnomePrintPdf *pdf, const char *content, gint len)
 }
 
 static gint
-gnome_print_pdf_page_fprintf (GnomePrintPdf *pdf, const char *format, ...)
+gnome_print_pdf_page_write (GnomePrintPdf *pdf, gchar *text)
 {
-	gint grow_size = GNOME_PRINT_PDF_INITIAL_BUFFER_SIZE;
-	GnomePrintPdfPage *page;
- 	va_list arguments;
- 	gchar *text;
-	gchar *oldlocale;
 	gint len;
+	GnomePrintPdfPage *page;
+	gint grow_size = GNOME_PRINT_PDF_INITIAL_BUFFER_SIZE;
 
-	/* Compose the text */
-	oldlocale = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
-#warning: We should not be using setlocale here
-
- 	va_start (arguments, format);
- 	text = g_strdup_vprintf (format, arguments);
- 	va_end (arguments);
-	setlocale (LC_NUMERIC, oldlocale);
-	g_free (oldlocale);
-	
-	/* Write it */
 	len = strlen (text);
 	page = pdf->pages->data;
 	if ((pdf->stream_used + len + 1) > pdf->stream_allocated) {
@@ -1804,11 +1866,50 @@ gnome_print_pdf_page_fprintf (GnomePrintPdf *pdf, const char *format, ...)
 	memcpy (pdf->stream + pdf->stream_used, text, len * sizeof (gchar));
 	pdf->stream_used += len;
 	pdf->stream [pdf->stream_used] = 0;
+
+	return len;
+}
+
+
+/* Note "format" should be locale independent, so it should not use %g */
+/* and friends */
+static gint
+gnome_print_pdf_page_fprintf (GnomePrintPdf *pdf, const char *format, ...)
+{
+ 	va_list arguments;
+	gint len;
+ 	gchar *text;
+
+	/* Compose the text */
+ 	va_start (arguments, format);
+ 	text = g_strdup_vprintf (format, arguments);
+ 	va_end (arguments);
+	
+	/* Write it */
+	len = gnome_print_pdf_page_write (pdf, text);
+
 	g_free (text);
 	
 	return len;
 }
 
+/* Allowed conversion specifiers are 'e', 'E', 'f', 'F', 'g' and 'G'. */
+static gint   
+gnome_print_pdf_page_print_double (GnomePrintPdf *pdf, 
+				   const gchar *format, gdouble x)
+{
+ 	gchar *text;
+	gint len;
+
+	text = g_new (gchar, G_ASCII_DTOSTR_BUF_SIZE);
+	g_ascii_formatd (text, G_ASCII_DTOSTR_BUF_SIZE, format, x);
+
+	len = gnome_print_pdf_page_write (pdf, text);
+	g_free (text);
+
+	return len;
+}
+
 static gint
 gnome_print_pdf_graphic_mode_set (GnomePrintPdf *pdf, PdfGraphicMode mode)
 {
