@@ -43,6 +43,11 @@ struct _GPAPrinter {
 	GPANode node;
 
 	gchar *name;
+
+	/* is_complete indicates whether the model and settings are set.   */
+	gboolean is_complete;
+	gchar *module_path;
+
 	GPANode *model;    /* of type GPAReference which points to a GPAModel */
 	GPANode *settings; /* of type GPAList, with childs of type GPASettings */
 };
@@ -52,6 +57,10 @@ GType     gpa_printer_get_type (void);
 
 GPANode * gpa_printer_new (const gchar *id, const gchar *name,
 			   GPAModel *model, GPASettings *settings);
+GPANode * gpa_printer_new_stub (const gchar *id, const gchar *name,
+				const gchar *);
+gboolean  gpa_printer_complete_stub (GPAPrinter *printer, 
+				     GPAModel *model, GPASettings *settings);
 
 GPAList * gpa_printer_list_load (void);
 
