@@ -87,6 +87,56 @@ xmlChar *model_xml_template =
 "  </Options>"
 "</Model>";
 
+/* Argument order: id */
+
+xmlChar *model_unknown_xml_template = 
+"<?xml version=\"1.0\"?>"
+"<Model Id=\"%s\" Version=\"1.0\">"
+"  <Name>Unavailable PPD File</Name>"
+"  <ModelVersion>0.0.1</ModelVersion>"
+"  <Options>"
+"    <Option Id=\"Transport\">"
+"      <Option Id=\"Backend\" Type=\"List\" Default=\"CUPS\">"
+"        <Item Id=\"CUPS\">"
+"          <Name>CUPS</Name>"
+"          <Key Id=\"Module\" Value=\"libgnomeprintcups.so\"/>"
+"        </Item>"
+"      </Option>"
+"    </Option>"
+"    <Option Id=\"Output\">"
+"      <Option Id=\"Media\">"
+"        <Option Id=\"PhysicalSize\" Type=\"List\" Default=\"USLetter\">"
+"          <Fill Ref=\"Globals.Media.PhysicalSize\"/>"
+"        </Option>"
+"        <Option Id=\"PhysicalOrientation\" Type=\"List\" Default=\"R0\">"
+"          <Fill Ref=\"Globals.Media.PhysicalOrientation\"/>"
+"        </Option>"
+"        <Key Id=\"Margins\">"
+"          <Key Id=\"Left\" Value=\"0\"/>"
+"          <Key Id=\"Right\" Value=\"0\"/>"
+"          <Key Id=\"Top\" Value=\"0\"/>"
+"          <Key Id=\"Bottom\" Value=\"0\"/>"
+"        </Key>"
+"      </Option>"
+"      <Option Id=\"Job\">"
+"        <Option Id=\"NumCopies\" Type=\"String\" Default=\"1\"/>"
+"        <Option Id=\"NonCollatedCopiesHW\" Type=\"String\" Default=\"true\"/>"
+"        <Option Id=\"CollatedCopiesHW\" Type=\"String\" Default=\"false\"/>"
+"        <Option Id=\"Collate\" Type=\"String\" Default=\"false\"/>"
+"        <Option Id=\"Duplex\" Type=\"String\" Default=\"true\"/>"
+"        <Option Id=\"Tumble\" Type=\"String\" Default=\"false\"/>"
+"        <Option Id=\"PrintToFile\" Type=\"String\" Default=\"false\" Locked=\"true\"/>"
+"        <Option Id=\"FileName\" Type=\"String\" Default=\"output.ps\"/>"
+"      </Option>"
+"    </Option>"
+#if 0
+"    <Option Id=\"Icon\">"
+"      <Option Id=\"Filename\" Type=\"String\" Default=\"" DATADIR "/pixmaps/nautilus/default/i-printer.png\"/>"
+"    </Option>"
+#endif
+"  </Options>"
+"</Model>";
+
 G_MODULE_EXPORT void gpa_module_load_data (GPAPrinter *printer);
 
 static GPANode *
@@ -342,6 +392,34 @@ get_model (const gchar *printer, ppd_file_t *ppd)
 	return (GPAModel *)model;
 }
 
+static GPAModel *
+get_model_no_ppd (const gchar *printer)
+{
+	GPANode *media;
+	GPANode *model;
+	GPANode *job;
+	char *xml;
+
+	model = gpa_model_get_by_id ("Cups-unknown-unknown", TRUE);
+	
+	if (model)
+		return GPA_MODEL (model);
+
+	xml = g_strdup_printf (model_unknown_xml_template, "Cups-unknown-unknown");
+	model = gpa_model_new_from_xml (xml);
+	g_free (xml);
+
+	media  = gpa_node_lookup (model, "Options.Output.Media");
+	job  = gpa_node_lookup (model, "Options.Output.Job");
+
+/* 	load_paper_sizes   (ppd, media); */
+	load_cups_hold_types (job);
+
+	gpa_node_unref (media);
+	gpa_node_unref (job);
+
+	return (GPAModel *)model;
+}
 
 static gboolean
 append_printer (GPAList *printers_list, const char *name, 
@@ -409,13 +487,16 @@ G_MODULE_EXPORT void gpa_module_load_data (GPAPrinter *printer)
 	if (printer->is_complete)
 		return;
 
-	filename = g_strdup (cupsGetPPD (name)); 
+	filename = g_strdup (cupsGetPPD (name));
 	ppd = ppdOpenFile (filename);
 
-	if (ppd == NULL)
-		goto gpa_module_load_data_exit;
-
-	model = get_model (name, ppd);
+	if (ppd != NULL) {
+		model = get_model (name, ppd);
+	} else {
+		g_warning ("The ppd file for the CUPS printer %s "
+			   "could not be loaded.", name);
+		model = get_model_no_ppd (name);
+	}
 	if (model == NULL)
 		goto gpa_module_load_data_exit;
 
